USE AdventureWorksDW2008XL;
GO

SELECT *
FROM   sys.column_store_segments;

SELECT     c.name                    AS "Column name",
           s.partition_id,
		   s.segment_id,
		   s.min_data_id,
		   s.max_data_id,
		   s.on_disk_size
FROM       sys.column_store_segments AS  s
INNER JOIN sys.partitions            AS  p
      ON   s.hobt_id                  =  p.hobt_id
INNER JOIN sys.indexes               AS  i 
      ON   i.object_id                =  p.object_id
INNER JOIN sys.columns               AS  c
      ON   c.object_id                =  p.object_id
	  AND  c.column_id                =  s.column_id
WHERE      i.type                     =  6
-- and     c.name                     = 'PromotionKey'
AND        i.name                     = 'CSI_FactResellerSalesXL'
ORDER BY   s.column_id,
           s.partition_id,
		   s.segment_id;

GO

SET STATISTICS IO ON;

-- Only a few segments have promotionkey values of over 15.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL
WHERE  PromotionKey > 15;

-- More segments have promotionkey values of over 12.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL
WHERE  PromotionKey > 12;

-- (Almost) all segments have promotionkey values of less than 12.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL
WHERE  PromotionKey < 12;

-- Due to the data distribution,
-- no segments can be eliminated based on ShipDateKey
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL
WHERE  ShipDateKey < 20030101;

-- Changing the WHERE clause to increase the time frame will hardly impact the amount of data read.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL
WHERE  ShipDateKey < 20030701;


-- Character columns do not support segment elimination,
-- so even though only a single segment would qualify here
-- (because SalesOrderNumber is the clustering key),
-- they are still all read.
-- (need a hint to force CS index for the demo;
--  otherwise the optimizer would -rightly- choose a clustered index seek)
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL WITH (INDEX = CSI_FactResellerSalesXL)
WHERE  SalesOrderNumber < 'SO43659-20020704-44'
GO







-- This database also has a very large FactResellerSales table,
-- but in this case, the clustered index is on OrderDateKey.
-- The data in the table is the same, but the schema is different:
-- foreign key constraints are left, but the primary key constraint,
-- all unique constraints, and all indexes are removed.
-- instead, a clustered index on OrderDateKey is created,
-- and then the columnstore index was built.
-- (This version of the table is also partitioned on OrderDateKey - more on that later)
USE AdventureWorksDW2008Big;
GO

-- Because DueDateKey and ShipDateKey are correlated to OrderDateKey,
-- their min_data_id and max_data_id per segment correlate to those of OrderDateKey as well.
SELECT     c.name                    AS "Column name",
           s.partition_id,
           s.segment_id,
		   s.min_data_id,
		   s.max_data_id,
		   s.on_disk_size
FROM       sys.column_store_segments AS  s
INNER JOIN sys.partitions            AS  p
      ON   s.hobt_id                  =  p.hobt_id
INNER JOIN sys.indexes               AS  i 
      ON   i.object_id                =  p.object_id
INNER JOIN sys.columns               AS  c
      ON   c.object_id                =  p.object_id
	  AND  c.column_id                =  s.column_id
WHERE      i.type                     =  6
AND        c.name                    IN('OrderDateKey', 'DueDateKey', 'ShipDateKey')
AND        i.name                     = 'CSI_FactResellerSales_Part'
ORDER BY   s.partition_id,
           s.segment_id,
		   s.column_id;
GO

-- Because of the high correlation between OrderDateKey and ShipDateKey,
-- many segments can now be eliminated based on ShipDateKey.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesPart
WHERE  ShipDateKey < 20030101;

-- Changing the WHERE clause to increase the time frame will increase the amount of data read.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesPart
WHERE  ShipDateKey < 20030701;
