USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

-- This is the query we used before.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimSalesTerritory      AS  st
      ON   st.SalesTerritoryKey        =  rs.SalesTerritoryKey
WHERE      st.SalesTerritoryCountry NOT IN('United States', 'Canada')
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- Let's see which SalesTerritoryKey values are good.
SELECT *
FROM   dbo.DimSalesTerritory;

-- Moving the filter to the fact table has very little impact on performance;
-- check the execution plans to understand why.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
INNER JOIN dbo.DimSalesTerritory      AS  st
      ON   st.SalesTerritoryKey        =  rs.SalesTerritoryKey
WHERE      rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

--DBCC DROPCLEANBUFFERS;

-- Obviously, the join to DimSalesTerritory is now no longer needed;
-- removing that does help a bit more with performance.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

--DBCC DROPCLEANBUFFERS;

-- The last optimization has nothing to do with CS indexes, though;
-- it also helps when using rowstore indexes.
-- (But it still is a lot slower than with the CS index)
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
INNER JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
WHERE      rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

-- Execution times with "cold" cache: 128296 ms vs 1634 ms = 78 x faster
-- Execution times with "hot" cache:    8968 ms vs  426 ms = 21 x faster


-- And here is another demonstration that segments can be eliminated
-- even if the search condition is on the smaller joined table.
-- First, check that PromotionKey > 15 equates StartDate > '20030701'.
SELECT *
FROM   dbo.DimPromotion
WHERE  StartDate > '20030701';
SELECT *
FROM   dbo.DimPromotion
WHERE  PromotionKey >= 15;

-- Now compare logical reads (for the fact table) of the two equivalent queries
SET STATISTICS TIME OFF;
SET STATISTICS IO ON;

SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL
WHERE  PromotionKey >= 15;

SELECT     COUNT(*)
FROM       dbo.FactResellerSalesXL AS rs
INNER JOIN dbo.DimPromotion AS p
      ON   p.PromotionKey = rs.PromotionKey
WHERE      p.StartDate > '20030701';
