USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

--DBCC DROPCLEANBUFFERS;

-- Same query as before, but now we want ALL products.
-- So we convert the INNER JOIN to a LEFT (outer) JOIN.
-- First, here's the version with rowstore indexes.
SELECT     p.EnglishProductName
		 , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
LEFT  JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
      AND  rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

--DBCC DROPCLEANBUFFERS;

-- And here is the equivalent with columnstore index.
SELECT     p.EnglishProductName
         , COUNT(rs.SalesOrderNumber) AS  NumberOfSales
         , SUM(rs.SalesAmount)        AS  TotalSalesAmount
FROM       dbo.DimProduct             AS  p
LEFT  JOIN dbo.FactResellerSalesXL    AS  rs
      ON   rs.ProductKey               =  p.ProductKey
      AND  rs.SalesTerritoryKey       >=  7
GROUP BY   p.EnglishProductName
ORDER BY   p.EnglishProductName;

-- Execution times with "cold" cache: 128392 ms vs 15511 ms = "only" 8.3 x faster
-- Execution times with "hot" cache:   28636 ms vs 11283 ms = "only" 2.5 x faster
