USE AdventureWorksDW2008Big;
GO

-- Attempt to modify the data
DELETE FROM dbo.FactResellerSalesPart
WHERE EmployeeKey = 17;
GO



/*
-- Disable the columnstore index
ALTER INDEX CSI_FactResellerSalesXL ON dbo.FactResellerSalesXL DISABLE;

-- Modify data
(...)

-- Rebuild the index - running time: 7:23 minutes!
ALTER INDEX CSI_FactResellerSalesXL ON dbo.FactResellerSalesXL REBUILD
*/


-- The partitioned version of the table is already created;
-- here is the code used to create it.
/*
CREATE PARTITION FUNCTION [ByOrderDateMonthPF](int)
  AS RANGE RIGHT
  FOR VALUES (20020701, 20020801, 20020901, 20021001, 20021101, 20021201,
              20030101, 20030201, 20030301, 20030401, 20030501, 20030601,
			  20030701, 20030801, 20030901, 20031001, 20031101, 20031201,
			  20040101, 20040201, 20040301, 20040401, 20040501, 20040601, 20040701);

CREATE PARTITION SCHEME [ByOrderDateMonthRange]
  AS PARTITION [ByOrderDateMonthPF]
  ALL TO ([PRIMARY]);

CREATE TABLE [dbo].[FactResellerSalesPart](
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[ResellerKey] [int] NOT NULL,
	[EmployeeKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NULL,
	[OrderQuantity] [smallint] NULL,
	[UnitPrice] [money] NULL,
	[ExtendedAmount] [money] NULL,
	[UnitPriceDiscountPct] [float] NULL,
	[DiscountAmount] [float] NULL,
	[ProductStandardCost] [money] NULL,
	[TotalProductCost] [money] NULL,
	[SalesAmount] [money] NULL,
	[TaxAmt] [money] NULL,
	[Freight] [money] NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL
) ON [ByOrderDateMonthRange]([OrderDateKey])
WITH (DATA_COMPRESSION = PAGE);
*/


-- **********************************************
--     MODIFYING DATA BY PARTITION SWITCHING
-- **********************************************

-- Method 1: Split empty partition,
-- then switch new data in one of the new partitions
-- (keeping the other one empty, for the next time)

-- Display data about the partitions
SELECT     p.partition_number, p.rows,
           prv.boundary_id, prv.value
FROM       sys.partition_range_values AS  prv
INNER JOIN sys.partition_functions    AS  pf
      ON   pf.function_id              =  prv.function_id
      AND  pf.name                     = 'ByOrderDateMonthPF'
RIGHT JOIN sys.partitions             AS  p
      ON   p.partition_number          =  prv.boundary_id
WHERE      p.object_id                 =  OBJECT_ID('FactResellerSalesPart')
AND        p.index_id                  =  1      -- Clustered index
ORDER BY   p.partition_number;


ALTER PARTITION SCHEME ByOrderDateMonthRange
NEXT USED [PRIMARY];

ALTER PARTITION FUNCTION ByOrderDateMonthPF ()
SPLIT RANGE (20020601);

-- Staging table must have the exact same schema,
-- except the columnstore index (to keep it read-write).
CREATE TABLE [dbo].[FactResellerSalesSTG](
	[ProductKey] [int] NOT NULL,
	[OrderDateKey] [int] NOT NULL,
	[DueDateKey] [int] NOT NULL,
	[ShipDateKey] [int] NOT NULL,
	[ResellerKey] [int] NOT NULL,
	[EmployeeKey] [int] NOT NULL,
	[PromotionKey] [int] NOT NULL,
	[CurrencyKey] [int] NOT NULL,
	[SalesTerritoryKey] [int] NOT NULL,
	[SalesOrderNumber] [nvarchar](20) NOT NULL,
	[SalesOrderLineNumber] [tinyint] NOT NULL,
	[RevisionNumber] [tinyint] NULL,
	[OrderQuantity] [smallint] NULL,
	[UnitPrice] [money] NULL,
	[ExtendedAmount] [money] NULL,
	[UnitPriceDiscountPct] [float] NULL,
	[DiscountAmount] [float] NULL,
	[ProductStandardCost] [money] NULL,
	[TotalProductCost] [money] NULL,
	[SalesAmount] [money] NULL,
	[TaxAmt] [money] NULL,
	[Freight] [money] NULL,
	[CarrierTrackingNumber] [nvarchar](25) NULL,
	[CustomerPONumber] [nvarchar](25) NULL
) ON [PRIMARY]
WITH
(
DATA_COMPRESSION = PAGE
)
GO

CREATE CLUSTERED INDEX [FactResellerSalesSTG_OrderDateKey_CI] ON [dbo].[FactResellerSalesSTG]
(
	[OrderDateKey] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON, DATA_COMPRESSION = PAGE) ON [PRIMARY]

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimCurrency] FOREIGN KEY([CurrencyKey])
REFERENCES [dbo].[DimCurrency] ([CurrencyKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimDate] FOREIGN KEY([OrderDateKey])
REFERENCES [dbo].[DimDate] ([DateKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimDate1] FOREIGN KEY([DueDateKey])
REFERENCES [dbo].[DimDate] ([DateKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimDate2] FOREIGN KEY([ShipDateKey])
REFERENCES [dbo].[DimDate] ([DateKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimEmployee] FOREIGN KEY([EmployeeKey])
REFERENCES [dbo].[DimEmployee] ([EmployeeKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimProduct] FOREIGN KEY([ProductKey])
REFERENCES [dbo].[DimProduct] ([ProductKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimPromotion] FOREIGN KEY([PromotionKey])
REFERENCES [dbo].[DimPromotion] ([PromotionKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimReseller] FOREIGN KEY([ResellerKey])
REFERENCES [dbo].[DimReseller] ([ResellerKey])

ALTER TABLE [dbo].[FactResellerSalesSTG]  WITH CHECK ADD  CONSTRAINT [FK_FactResellerSalesSTG_DimSalesTerritory] FOREIGN KEY([SalesTerritoryKey])
REFERENCES [dbo].[DimSalesTerritory] ([SalesTerritoryKey])


-- Get some data in the staging table
INSERT INTO dbo.FactResellerSalesSTG
SELECT TOP (5) * FROM dbo.FactResellerSalesPart
ORDER BY OrderDateKey;
UPDATE dbo.FactResellerSalesSTG
SET    OrderDateKey = 20020625
SELECT *
FROM   dbo.FactResellerSalesSTG;


-- Now try to switch the staging table into the main table.
-- This will fail at this time.

-- Get partition number to use
DECLARE @Part int = $PARTITION.ByOrderDateMonthPF(20020625);

ALTER TABLE dbo.FactResellerSalesSTG
SWITCH TO dbo.FactResellerSalesPart PARTITION @Part;
GO

-- Build a columnstore index on the staging table.
CREATE NONCLUSTERED COLUMNSTORE INDEX [CSI_FactResellerSalesSTG] ON [dbo].[FactResellerSalesSTG]
(
	[ProductKey],
	[OrderDateKey],
	[DueDateKey],
	[ShipDateKey],
	[ResellerKey],
	[EmployeeKey],
	[PromotionKey],
	[CurrencyKey],
	[SalesTerritoryKey],
	[SalesOrderNumber],
	[SalesOrderLineNumber],
	[RevisionNumber],
	[OrderQuantity],
	[UnitPrice],
	[ExtendedAmount],
	[UnitPriceDiscountPct],
	[DiscountAmount],
	[ProductStandardCost],
	[TotalProductCost],
	[SalesAmount],
	[TaxAmt],
	[Freight],
	[CarrierTrackingNumber],
	[CustomerPONumber]
)WITH (DROP_EXISTING = OFF) ON [PRIMARY]

-- Try to switch the staging table into the main table again.
-- This will still fail, but for a different reason.
DECLARE @Part int = $PARTITION.ByOrderDateMonthPF(20020625);
ALTER TABLE dbo.FactResellerSalesSTG
SWITCH TO dbo.FactResellerSalesPart PARTITION @Part;
GO

-- Add the constraint that ensures that all data is in the correct range.
ALTER TABLE dbo.FactResellerSalesSTG
WITH CHECK
ADD CONSTRAINT CK_Correct_Partition
    CHECK (OrderDateKey >= 20020601 AND OrderDateKey < 20020701);

-- Switching the staging table into the main table should now work.
DECLARE @Part int = $PARTITION.ByOrderDateMonthPF(20020625);
ALTER TABLE dbo.FactResellerSalesSTG
SWITCH TO dbo.FactResellerSalesPart PARTITION @Part;
GO

-- Display data about the partitions
SELECT     p.partition_number, p.rows,
           prv.boundary_id, prv.value
FROM       sys.partition_range_values AS  prv
INNER JOIN sys.partition_functions    AS  pf
      ON   pf.function_id              =  prv.function_id
      AND  pf.name                     = 'ByOrderDateMonthPF'
RIGHT JOIN sys.partitions             AS  p
      ON   p.partition_number          =  prv.boundary_id
WHERE      p.object_id                 =  OBJECT_ID('FactResellerSalesPart')
AND        p.index_id                  =  1      -- Clustered index
ORDER BY   p.partition_number;


-- **********************************************
--     MODIFYING DATA BY PARTITION SWITCHING
-- **********************************************

-- Method 2: Switch out, modify, then switch back in.

-- Splitting a non-empty partition is not allowed.
ALTER PARTITION SCHEME ByOrderDateMonthRange
NEXT USED [PRIMARY];

ALTER PARTITION FUNCTION ByOrderDateMonthPF ()
SPLIT RANGE (20040801);

-- When we need to split a non-empty partition,
-- or when we simply need to modify existing data,
-- we need to use a different method.


-- The staging table still exists, but is empty again.
SELECT *
FROM   dbo.FactResellerSalesSTG;

-- First, change the CHECK constraint to reflect the new partition range
ALTER TABLE dbo.FactResellerSalesSTG
DROP CONSTRAINT CK_Correct_Partition;

ALTER TABLE dbo.FactResellerSalesSTG
WITH CHECK
ADD CONSTRAINT CK_Correct_Partition
    CHECK (OrderDateKey >= 20040701);

-- Now switch out the partition to be modified

-- Get partition number to use
DECLARE @Part int = $PARTITION.ByOrderDateMonthPF(20040701);

ALTER TABLE dbo.FactResellerSalesPart
SWITCH PARTITION @Part TO dbo.FactResellerSalesSTG;
GO

-- Note that the partition just switched out is now empty;
-- when needed, split it now, before switching data back in later.
-- If you do this, remember that the data in the staging table
-- may need to be divided over the new partitions;
-- demonstrating the code for this is beyond the scope of this presentation.

-- Disable the columnstore index to allow modification,
-- then make the changes, and rebuild the CS index.
ALTER INDEX CSI_FactResellerSalesSTG ON dbo.FactResellerSalesSTG DISABLE;

DELETE TOP(500000) FROM dbo.FactResellerSalesSTG;

ALTER INDEX CSI_FactResellerSalesSTG ON dbo.FactResellerSalesSTG REBUILD

-- Now the modified data can be switched in again.
DECLARE @Part int = $PARTITION.ByOrderDateMonthPF(20040701);

ALTER TABLE dbo.FactResellerSalesSTG
SWITCH TO dbo.FactResellerSalesPart PARTITION @Part;
GO


-- Display data about the partitions
SELECT     p.partition_number, p.rows,
           prv.boundary_id, prv.value
FROM       sys.partition_range_values AS  prv
INNER JOIN sys.partition_functions    AS  pf
      ON   pf.function_id              =  prv.function_id
      AND  pf.name                     = 'ByOrderDateMonthPF'
RIGHT JOIN sys.partitions             AS  p
      ON   p.partition_number          =  prv.boundary_id
WHERE      p.object_id                 =  OBJECT_ID('FactResellerSalesPart')
AND        p.index_id                  =  1      -- Clustered index
ORDER BY   p.partition_number;
GO
