USE AdventureWorksDW2008XL;
GO

SET STATISTICS TIME ON;

-- Aggregates without GROUP BY - how simple do you want your query to be?
-- And yet, the query plan uses row mode instead of batch mode.
SELECT COUNT(*)
FROM   dbo.FactResellerSalesXL;


-- Microsoft has said they're working to change this.
-- But for now, here is a possible workaround:
-- First compute the aggregate using a GROUP BY,
-- then compute the final aggregate.
-- Based on a few tests, I expect the best performance
-- if you GROUP BY a column with few distinct values.
-- SalesTerritoryKey has only 10 distinct values: fast
SELECT SUM(cnt)
FROM  (SELECT   COUNT(*) AS cnt
       FROM     dbo.FactResellerSalesXL
	   GROUP BY SalesTerritoryKey) AS d;

-- OrderDateKey has 725 distinct values - not quite so fast.
SELECT SUM(cnt)
FROM  (SELECT   COUNT(*) AS cnt
       FROM     dbo.FactResellerSalesXL
	   GROUP BY OrderDateKey) AS d;


-- Since fewer groups in the inner query speed up the query,
-- let's try to trick the optimizer - the one below will always result in one group!
SELECT SUM(cnt)
FROM  (SELECT   COUNT(*) AS cnt
       FROM     dbo.FactResellerSalesXL
	   GROUP BY SalesTerritoryKey * 0) AS d;

-- But the column chosen is still relevant!
-- This query is more expensive, because bad estimates result in pointless parallellism
SELECT SUM(cnt)
FROM  (SELECT   COUNT(*) AS cnt
       FROM     dbo.FactResellerSalesXL
	   GROUP BY OrderDateKey * 0) AS d;
